//
//  KeyView.swift
//  OTP
//
//  Created by Shadowfacts on 8/21/21.
//

import SwiftUI
import OTPKit

struct KeyView: View {
    let key: TOTPKey
    let currentCode: TOTPCode
    
    var formattedCode: String {
        let code = currentCode.code
        let mid = code.index(code.startIndex, offsetBy: code.count / 2)
        return "\(code[code.startIndex..<mid]) \(code[mid...])"
    }
    
    init(key: TOTPKey, currentCode: TOTPCode) {
        self.key = key
        self.currentCode = currentCode
    }
    
    var body: some View {
        HStack {
            VStack(alignment: .leading) {
                Text(key.issuer)
                    .font(.title3)
                
                if let label = key.label, !label.isEmpty {
                    Text(label)
                        .font(.footnote)
                }
            }
            
            Spacer()
            
            Text(formattedCode)
                .font(.system(.title2, design: .monospaced))
            
//            Text("\(currentCode.validUntil, style: .relative)")
//                .font(.body.monospacedDigit())
            
            // I don't think this TimelineView should be necessary since the CodeHolder timer fires every .5 seconds
            TimelineView(.animation) { (ctx) in
                ZStack {
                    CircularProgressView(progress: progress(at: Date()), colorChangeThreshold: 5.0 / Double(key.period))
                    
                    Text(Int(round(currentCode.validUntil.timeIntervalSinceNow)).description)
                        .font(.caption.monospacedDigit())
                }
                .frame(width: 30)
            }
            
        }
    }
    
    private func progress(at date: Date) -> Double {
        let seconds = round(date.timeIntervalSince(currentCode.validFrom))
        let progress = 1 - seconds / Double(key.period)
        return progress
    }
}

struct KeyView_Previews: PreviewProvider {
    static var key: TOTPKey {
        TOTPKey(urlComponents: URLComponents(string: "otpauth://totp/Example:alice@google.com?secret=JBSWY3DPEHPK3PXP&issuer=Example")!)!
    }
    static var code: TOTPCode {
        OTPGenerator.generate(key: key)
    }
    static var previews: some View {
        KeyView(key: key, currentCode: code)
    }
}
